import argparse
import shipyard_bp_utils as shipyard
import sys
from shipyard_snowflake import SnowflakeClient
from shipyard_templates import ExitCodeException, ShipyardLogger

logger = ShipyardLogger.get_logger()


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--username", dest="username", required=True)
    parser.add_argument("--password", dest="password", required=False)
    parser.add_argument(
        "--private-key-path", dest="private_key_path", required=False, default=""
    )
    parser.add_argument(
        "--private-key-passphrase", dest="private_key_passphrase", required=False
    )
    parser.add_argument("--account", dest="account", required=True)
    parser.add_argument("--warehouse", dest="warehouse", required=False)
    parser.add_argument("--database", dest="database", required=True)
    parser.add_argument("--schema", dest="schema", required=False)
    parser.add_argument("--query", dest="query", required=True)
    parser.add_argument(
        "--destination-file-name",
        dest="destination_file_name",
        default="output.csv",
        required=True,
    )
    parser.add_argument(
        "--destination-folder-name",
        dest="destination_folder_name",
        default="",
        required=False,
    )
    parser.add_argument(
        "--file-header", dest="file_header", default="True", required=False
    )
    parser.add_argument("--user-role", dest="user_role", required=False, default="")
    return parser.parse_args()


def main():
    destination_full_path = None
    try:
        args = get_args()
        client_args = {
            "username": args.username,
            "password": None if args.password == "" else args.password,
            "account": None if args.account == "" else args.account,
            "warehouse": args.warehouse,
            "schema": args.schema,
            "database": args.database,
            "rsa_key": None if args.private_key_path == "" else args.private_key_path,
            "role": None if args.user_role == "" else args.user_role,
        }
        private_key_passphrase = (
            None if args.private_key_passphrase == "" else args.private_key_passphrase
        )

        destination_file_name = args.destination_file_name
        destination_folder_name = shipyard.files.clean_folder_name(
            args.destination_folder_name or ''
        )
        shipyard.files.create_folder_if_dne(destination_folder_name)
        destination_full_path = shipyard.files.combine_folder_and_file_name(
            folder_name=destination_folder_name, file_name=destination_file_name
        )
        file_header = shipyard.args.convert_to_boolean(args.file_header)
        client = SnowflakeClient(**client_args)

        # check to make sure that if a private key is provided, a passphrase is also provided
        if client.rsa_key and not private_key_passphrase:
            logger.error(
                "Error: A private key passphrase must be provided if using a private key"
            )
            sys.exit(client.EXIT_CODE_INVALID_ARGUMENTS)

        client.connect()
        logger.debug(f"Provided query is {args.query}")
        df = client.fetch(args.query)

        if df.empty:
            logger.error("No results returned from query")
            sys.exit(client.EXIT_CODE_NO_RESULTS)

        logger.debug(f"Shape of dataframe is {df.shape}")
        df.to_csv(destination_full_path, index=False, header=file_header)
        logger.info(f"Successfully saved query results to {destination_full_path}")
    except ExitCodeException as e:
        logger.error(e.message)
        sys.exit(e.exit_code)
    except Exception as e:
        logger.error(f"Error writing file to {destination_full_path}")
        logger.error(str(e))
        sys.exit(SnowflakeClient.EXIT_CODE_NO_RESULTS)
    else:
        logger.info("Closing Connection")


if __name__ == "__main__":
    main()



import pandas as pd
from shipyard_templates import ExitCodeException, Database, ShipyardLogger
from snowflake.snowpark.session import Session

logger = ShipyardLogger.get_logger()


class SnowparkClient(Database):
    def __init__(
        self,
        username,
        pwd,
        database=None,
        account=None,
        warehouse=None,
        schema=None,
        role=None,
    ):
        self.username = username
        self.pwd = pwd
        self.account = account
        self.database = database
        self.warehouse = warehouse
        self.schema = schema
        self.role = role

        super().__init__(
            username,
            pwd,
            account=account,
            warehouse=warehouse,
            schema=schema,
            database=database,
        )

    def connect(self):
        params = {
            "account": self.account,
            "user": self.username,
            "password": self.pwd,
            "warehouse": self.warehouse,
            "database": self.database,
            "schema": self.schema,
            "role": self.role,
        }

        try:
            session = Session.builder.configs(params).create()
        except Exception as e:
            logger.error(f"Error from snowpark {e}")
            raise ExitCodeException(
                message="Could not establish Snowpark Session",
                exit_code=self.EXIT_CODE_INVALID_CREDENTIALS,
            )
        else:
            return session

    def upload(
        self, session: Session, df: pd.DataFrame, table_name: str, overwrite=True
    ):
        """
        Uploads a pandas dataframe to a table in Snowflake using the Snowpark API

        Args:
            overwrite (): Whether to overwrite the existing table. True to replace, False to append to table (if it exists)
            session: The Snowpark Session established by the connect method
            df: The pandas dataframe to load
            table_name: The name of the Snowflake Table to write to
        """

        logger.info("Attempting to write pandas dataframe to Snowflake")
        try:
            session.write_pandas(
                df, table_name=table_name, auto_create_table=True, overwrite=overwrite
            )
            logger.info(f"Successfully wrote data to {table_name}")

        except Exception as e:
            logger.error(f"Error in writing data to snowflake: {e}")
            raise ExitCodeException(
                message=e, exit_code=self.EXIT_CODE_INVALID_UPLOAD_VALUE
            )

    def put(self, session: Session, file_path: str, table_name: str, overwrite=False):
        stage_name = f'@%"{table_name}"'
        if overwrite:
            try:
                session.sql(f"CREATE OR REPLACE stage {table_name}").collect()
                logger.info("Successfully created table in Snowflake")
            except Exception as e:
                logger.error("Error in creating table")
                raise ExitCodeException(
                    message=e, exit_code=self.EXIT_CODE_INVALID_UPLOAD_VALUE
                )
        logger.info("Attempting to put file to Snowflake")
        try:
            session.file.put(file_path, stage_name, overwrite=overwrite)
            logger.info("Successfully loaded file into Snowflake")
        except Exception as e:
            logger.error("Error in putting file in Snowflake")
            raise ExitCodeException(
                message=e, exit_code=self.EXIT_CODE_INVALID_UPLOAD_VALUE
            )

    def copy_into(self, session: Session, table_name: str):
        logger.info("Attempting to copy into Snowflake")
        try:
            copy_statement = f"""COPY INTO "{table_name}" FROM '@%\"{table_name}\"' PURGE=TRUE FILE_FORMAT=(TYPE=CSV FIELD_DELIMITER=',' COMPRESSION=GZIP, PARSE_HEADER=TRUE) MATCH_BY_COLUMN_NAME=CASE_INSENSITIVE"""
            session.sql(copy_statement)
            logger.info("Successfully copied into Snowflake")
        except Exception as e:
            logger.error("Error in copying into Snowflake")
            raise ExitCodeException(
                message=e, exit_code=self.EXIT_CODE_INVALID_UPLOAD_VALUE
            )

    def execute_query(self):
        pass

    def fetch(self):
        pass
